import bpy
import gpu
import blf
from gpu_extras.batch import batch_for_shader
from gpu_extras.presets import draw_circle_2d
import time

# Store the current drawing data
draw_data = {
    "text": "",
    "start_time": 0,
    "duration": 2.0,  # Display duration in seconds
    "fade_duration": 1.0,  # Fade out duration in seconds
    "color": [1.0, 1.0, 1.0, 1.0],  # RGBA
    "size": 24,
    "position": (100, 100),  # Default position
    "vertical_position": 90.0,  # Default vertical position (percentage from bottom)
    "horizontal_position": 50.0,  # Default horizontal position (percentage from left)
    "handler": None,
    "is_enabled": False,
    "timer": None,  # Timer for continuous redraws
}

def draw_callback_px():
    """Draw the text in the viewport"""
    # Calculate time elapsed
    elapsed = time.time() - draw_data["start_time"]
    
    # If the display duration has been exceeded, remove the handler
    if elapsed > draw_data["duration"] + draw_data["fade_duration"]:
        _remove_handler()
        return
    
    # Calculate alpha based on fade timing
    if elapsed > draw_data["duration"]:
        # Fade out phase
        fade_progress = (elapsed - draw_data["duration"]) / draw_data["fade_duration"]
        alpha = 1.0 - fade_progress
    else:
        alpha = 1.0
    
    # Set text properties
    font_id = 0  # Default font
    blf.position(font_id, draw_data["position"][0], draw_data["position"][1], 0)
    blf.size(font_id, draw_data["size"])
    
    # Set color with current alpha
    color = draw_data["color"].copy()
    color[3] = alpha * color[3]
    blf.color(font_id, *color)
    
    # Draw the text
    blf.draw(font_id, draw_data["text"])

def timer_update():
    """Timer callback to force redraw during fading"""
    # Force a redraw of all 3D VIEW regions
    for window in bpy.context.window_manager.windows:
        for area in window.screen.areas:
            if area.type == 'VIEW_3D':
                for region in area.regions:
                    if region.type == 'WINDOW':
                        region.tag_redraw()
    
    # Calculate time elapsed
    elapsed = time.time() - draw_data["start_time"]
    
    # Stop the timer if we're done fading
    if elapsed > draw_data["duration"] + draw_data["fade_duration"]:
        _remove_timer()
        return None
    
    return 0.01  # 100 fps refresh rate

def _remove_handler():
    """Internal function to remove the draw handler if it exists"""
    if draw_data["handler"] is not None:
        bpy.types.SpaceView3D.draw_handler_remove(draw_data["handler"], 'WINDOW')
        draw_data["handler"] = None
        draw_data["is_enabled"] = False
    
    # Also remove the timer
    _remove_timer()

def _remove_timer():
    """Internal function to remove the timer if it exists"""
    if draw_data["timer"] is not None:
        bpy.app.timers.remove(draw_data["timer"])
        draw_data["timer"] = None

def remove_handler():
    """Public function to remove the draw handler, for use by other modules"""
    _remove_handler()

def update_text(text, position=None):
    """Update the text to be displayed in the viewport"""
    # Check if viewport feedback is enabled in preferences
    try:
        addon_prefs = bpy.context.preferences.addons[__package__].preferences
        if not addon_prefs.display_viewport_feedback:
            # Viewport feedback is disabled, don't show any text
            return
    except (AttributeError, KeyError):
        # If we can't access preferences, default to enabled
        pass
    
    # If there's already an active handler, remove it
    if draw_data["is_enabled"]:
        _remove_handler()
    
    # Update the draw data
    draw_data["text"] = text
    draw_data["start_time"] = time.time()
    
    if position:
        draw_data["position"] = position
    else:
        # Get the region dimensions
        region = bpy.context.region
        if region:
            # Calculate vertical position based on preference (percentage from bottom)
            vertical_pos = int(region.height * (draw_data["vertical_position"] / 100.0))
            
            # Calculate horizontal position based on preference (percentage from left)
            text_width = int(len(text) * draw_data["size"] * 0.50)  # Approximate text width
            horizontal_pos = int(region.width * (draw_data["horizontal_position"] / 100.0)) - (text_width // 2)
            
            draw_data["position"] = (horizontal_pos, vertical_pos)
    
    # Add the draw handler
    if not draw_data["is_enabled"]:
        draw_data["handler"] = bpy.types.SpaceView3D.draw_handler_add(
            draw_callback_px, (), 'WINDOW', 'POST_PIXEL')
        draw_data["is_enabled"] = True
    
    # Start the timer for continuous redraws
    if draw_data["timer"] is None:
        draw_data["timer"] = bpy.app.timers.register(timer_update)
    
    # Force initial redraw of all 3D VIEW regions
    for window in bpy.context.window_manager.windows:
        for area in window.screen.areas:
            if area.type == 'VIEW_3D':
                for region in area.regions:
                    if region.type == 'WINDOW':
                        region.tag_redraw()

def show_scene_name(context):
    """Display the current scene name in the viewport"""
    scene = context.scene
    scene_name = scene.name
    
    # Get the icon for the current Blender version
    icon_text = "🎬 "  # Fallback to emoji as a universal option
    
    # Add excluded indicator if applicable
    excluded_indicator = " [Excluded]" if not scene.include_in_cycle else ""
    
    # Update the text with the icon prefix
    update_text(f"{icon_text}{scene_name}{excluded_indicator}")

def show_view_layer_name(context):
    """Display the current view layer name in the viewport"""
    view_layer = context.view_layer
    view_layer_name = view_layer.name
    
    # Fallback to emoji as a universal option
    icon_text = "👁️ "
    
    # Add excluded indicator if applicable
    excluded_indicator = " [Excluded]" if not view_layer.include_in_cycle else ""
    
    # Update the text with the icon prefix
    update_text(f"{icon_text}{view_layer_name}{excluded_indicator}")

def register():
    # Nothing to register here
    pass

def unregister():
    # Make sure to remove any active handlers on addon unregister
    _remove_handler() 