import bpy
from bpy.types import Operator
from bpy.utils import register_class, unregister_class
from bpy.props import BoolProperty
from . import draw_handler

# Add properties to scenes and view layers
bpy.types.Scene.include_in_cycle = BoolProperty(
    name="Include in Switcher",
    description="When enabled, this scene will be included when cycling through scenes. Disable to skip this scene when using next/previous shortcuts",
    default=True
)

bpy.types.ViewLayer.include_in_cycle = BoolProperty(
    name="Include in Switcher",
    description="When enabled, this view layer will be included when cycling through view layers. Disable to skip this view layer when using next/previous shortcuts",
    default=True
)

def switch_to_next_scene(context):
    current_scene = context.scene
    scenes = bpy.data.scenes
    scene_names = [scene.name for scene in scenes]
    current_index = scene_names.index(current_scene.name)
    
    # Safety check - if there's only one scene or all scenes are excluded, do nothing
    if len(scenes) <= 1:
        draw_handler.show_scene_name(context)
        return
        
    # Count included scenes
    included = sum(1 for scene in scenes if scene.include_in_cycle)
    if included == 0:
        # All scenes are excluded, just go to the next one anyway
        next_index = (current_index + 1) % len(scenes)
        context.window.scene = scenes[next_index]
        draw_handler.show_scene_name(context)
        return
    
    # Find the next included scene
    next_index = current_index
    for _ in range(len(scenes)):
        next_index = (next_index + 1) % len(scenes)
        next_scene = scenes[next_index]
        if next_scene.include_in_cycle:
            context.window.scene = next_scene
            break
            
    # Show the new scene name in the viewport
    draw_handler.show_scene_name(context)

def switch_to_previous_scene(context):
    current_scene = context.scene
    scenes = bpy.data.scenes
    scene_names = [scene.name for scene in scenes]
    current_index = scene_names.index(current_scene.name)
    
    # Safety check - if there's only one scene or all scenes are excluded, do nothing
    if len(scenes) <= 1:
        draw_handler.show_scene_name(context)
        return
        
    # Count included scenes
    included = sum(1 for scene in scenes if scene.include_in_cycle)
    if included == 0:
        # All scenes are excluded, just go to the previous one anyway
        previous_index = (current_index - 1) % len(scenes)
        context.window.scene = scenes[previous_index]
        draw_handler.show_scene_name(context)
        return
    
    # Find the previous included scene
    previous_index = current_index
    for _ in range(len(scenes)):
        previous_index = (previous_index - 1) % len(scenes)
        previous_scene = scenes[previous_index]
        if previous_scene.include_in_cycle:
            context.window.scene = previous_scene
            break
            
    # Show the new scene name in the viewport
    draw_handler.show_scene_name(context)

def switch_to_next_view_layer(context):
    current_view_layer = context.view_layer
    view_layers = context.scene.view_layers
    vl_names = [vl.name for vl in view_layers]
    current_index = vl_names.index(current_view_layer.name)
    
    # Safety check - if there's only one view layer or all view layers are excluded, do nothing
    if len(view_layers) <= 1:
        draw_handler.show_view_layer_name(context)
        return
        
    # Count included view layers
    included = sum(1 for vl in view_layers if vl.include_in_cycle)
    if included == 0:
        # All view layers are excluded, just go to the next one anyway
        next_index = (current_index + 1) % len(view_layers)
        context.window.view_layer = view_layers[next_index]
        draw_handler.show_view_layer_name(context)
        return
    
    # Find the next included view layer
    next_index = current_index
    for _ in range(len(view_layers)):
        next_index = (next_index + 1) % len(view_layers)
        next_view_layer = view_layers[next_index]
        if next_view_layer.include_in_cycle:
            context.window.view_layer = next_view_layer
            break
            
    # Show the new view layer name in the viewport
    draw_handler.show_view_layer_name(context)

def switch_to_previous_view_layer(context):
    current_view_layer = context.view_layer
    view_layers = context.scene.view_layers
    vl_names = [vl.name for vl in view_layers]
    current_index = vl_names.index(current_view_layer.name)
    
    # Safety check - if there's only one view layer or all view layers are excluded, do nothing
    if len(view_layers) <= 1:
        draw_handler.show_view_layer_name(context)
        return
        
    # Count included view layers
    included = sum(1 for vl in view_layers if vl.include_in_cycle)
    if included == 0:
        # All view layers are excluded, just go to the previous one anyway
        previous_index = (current_index - 1) % len(view_layers)
        context.window.view_layer = view_layers[previous_index]
        draw_handler.show_view_layer_name(context)
        return
    
    # Find the previous included view layer
    previous_index = current_index
    for _ in range(len(view_layers)):
        previous_index = (previous_index - 1) % len(view_layers)
        previous_view_layer = view_layers[previous_index]
        if previous_view_layer.include_in_cycle:
            context.window.view_layer = previous_view_layer
            break
            
    # Show the new view layer name in the viewport
    draw_handler.show_view_layer_name(context)

class SCENE_SWITCHER_OT_next_scene(Operator):
    """Switch to the next scene"""
    bl_idname = "scene_switcher.next_scene"
    bl_label = "Next Scene"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        switch_to_next_scene(context)
        return {'FINISHED'}

class SCENE_SWITCHER_OT_previous_scene(Operator):
    """Switch to the previous scene"""
    bl_idname = "scene_switcher.previous_scene"
    bl_label = "Previous Scene"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        switch_to_previous_scene(context)
        return {'FINISHED'}

class SCENE_SWITCHER_OT_next_view_layer(Operator):
    """Switch to the next view layer"""
    bl_idname = "scene_switcher.next_view_layer"
    bl_label = "Next View Layer"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        switch_to_next_view_layer(context)
        return {'FINISHED'}

class SCENE_SWITCHER_OT_previous_view_layer(Operator):
    """Switch to the previous view layer"""
    bl_idname = "scene_switcher.previous_view_layer"
    bl_label = "Previous View Layer"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        switch_to_previous_view_layer(context)
        return {'FINISHED'}

classes = (
    SCENE_SWITCHER_OT_next_scene,
    SCENE_SWITCHER_OT_previous_scene,
    SCENE_SWITCHER_OT_next_view_layer,
    SCENE_SWITCHER_OT_previous_view_layer,
)

def register():
    for cls in classes:
        register_class(cls)

def unregister():
    for cls in reversed(classes):
        unregister_class(cls)
