import bpy
import rna_keymap_ui
from . import draw_handler

class SceneSwitcherPreferences(bpy.types.AddonPreferences):
    bl_idname = __package__

    # Global setting to enable/disable viewport feedback
    display_viewport_feedback: bpy.props.BoolProperty(
        name="Display Viewport Feedback",
        description="Enable/disable text feedback in the 3D viewport when switching scenes or view layers",
        default=True,
        update=lambda self, context: self.update_viewport_feedback_state()
    )
    
    # Viewport text display preferences
    text_size: bpy.props.IntProperty(
        name="Text Size",
        description="Size of the viewport text",
        default=24,
        min=10,
        max=50,
        update=lambda self, context: self.update_draw_handler_settings()
    )
    
    display_duration: bpy.props.FloatProperty(
        name="Display Duration",
        description="How long to display the text (seconds)",
        default=0.5,
        min=0.5,
        max=10.0,
        update=lambda self, context: self.update_draw_handler_settings()
    )
    
    fade_duration: bpy.props.FloatProperty(
        name="Fade Duration",
        description="How long to fade out the text (seconds)",
        default=1.0,
        min=0.1,
        max=5.0,
        update=lambda self, context: self.update_draw_handler_settings()
    )
    
    text_color: bpy.props.FloatVectorProperty(
        name="Text Color",
        description="Color of the viewport text",
        default=(1.0, 1.0, 1.0, 1.0),
        size=4,
        min=0.0,
        max=1.0,
        subtype='COLOR',
        update=lambda self, context: self.update_draw_handler_settings()
    )
    
    vertical_position: bpy.props.FloatProperty(
        name="Vertical Position",
        description="Vertical position of the text (percentage from bottom)",
        default=90.0,  # Default near the top of the viewport
        min=0.0,
        max=100.0,
        subtype='PERCENTAGE',
        update=lambda self, context: self.update_draw_handler_settings()
    )
    
    horizontal_position: bpy.props.FloatProperty(
        name="Horizontal Position",
        description="Horizontal position of the text (percentage from left)",
        default=50.0,  # Default centered in the viewport
        min=0.0,
        max=100.0,
        subtype='PERCENTAGE',
        update=lambda self, context: self.update_draw_handler_settings()
    )

    def update_viewport_feedback_state(self):
        """Update the viewport feedback state based on the preference"""
        # If disabled, make sure to remove any active viewport text
        if not self.display_viewport_feedback:
            draw_handler.remove_handler()

    def update_draw_handler_settings(self):
        """Update the draw handler settings when preferences change"""
        draw_handler.draw_data["size"] = self.text_size
        draw_handler.draw_data["duration"] = self.display_duration
        draw_handler.draw_data["fade_duration"] = self.fade_duration
        draw_handler.draw_data["color"] = list(self.text_color)
        draw_handler.draw_data["vertical_position"] = self.vertical_position
        draw_handler.draw_data["horizontal_position"] = self.horizontal_position

    def draw(self, context):
        layout = self.layout
        
        # Global viewport feedback toggle
        layout.prop(self, "display_viewport_feedback")
        
        # Text display settings
        box = layout.box()
        box.label(text="Viewport Text Display Settings:")
        
        # Enable/disable the box based on the display_viewport_feedback setting
        box.enabled = self.display_viewport_feedback
        
        row = box.row()
        row.prop(self, "text_size")
        row.prop(self, "text_color")
        
        row = box.row()
        row.prop(self, "display_duration")
        row.prop(self, "fade_duration")
        
        row = box.row()
        row.prop(self, "vertical_position")
        row.prop(self, "horizontal_position")
        
        # Keymaps
        layout.separator()
        self.draw_keymaps(context, layout)

    def draw_keymaps(self, context, layout):
        wm = context.window_manager
        kc = wm.keyconfigs.user

        keymaps_info = [
            {
                'km_name': 'Window',
                'space_type': 'EMPTY',
                'region_type': 'WINDOW',
                'operator_idname': 'scene_switcher.next_scene',
            },
            {
                'km_name': 'Window',
                'space_type': 'EMPTY',
                'region_type': 'WINDOW',
                'operator_idname': 'scene_switcher.previous_scene',
            },
            {
                'km_name': 'Window',
                'space_type': 'EMPTY',
                'region_type': 'WINDOW',
                'operator_idname': 'scene_switcher.next_view_layer',
            },
            {
                'km_name': 'Window',
                'space_type': 'EMPTY',
                'region_type': 'WINDOW',
                'operator_idname': 'scene_switcher.previous_view_layer',
            },
        ]

        # Create a column layout with alignment
        col = layout.column(align=True)
        col.label(text="Keyboard Shortcuts:")
        for km_info in keymaps_info:
            km = kc.keymaps.get(km_info['km_name'])
            if not km:
                continue

            for kmi in km.keymap_items:
                if kmi.idname == km_info['operator_idname']:
                    # Draw the keymap item without labels and minimize spacing
                    rna_keymap_ui.draw_kmi([], kc, km, kmi, col, 0)
                    break

def register():
    bpy.utils.register_class(SceneSwitcherPreferences)
    
    # Initialize draw handler settings from preferences
    addon_prefs = bpy.context.preferences.addons[__package__].preferences
    if addon_prefs:
        # Update the draw handler settings
        addon_prefs.update_draw_handler_settings()
        
        # Check if viewport feedback should be disabled initially
        if not addon_prefs.display_viewport_feedback:
            draw_handler.remove_handler()
    else:
        # If preferences not available, use defaults
        from . import draw_handler
        draw_handler.draw_data["size"] = 24
        draw_handler.draw_data["duration"] = 0.5
        draw_handler.draw_data["fade_duration"] = 1.0
        draw_handler.draw_data["color"] = [1.0, 1.0, 1.0, 1.0]
        draw_handler.draw_data["vertical_position"] = 90.0
        draw_handler.draw_data["horizontal_position"] = 50.0

def unregister():
    bpy.utils.unregister_class(SceneSwitcherPreferences)
